# --------------------------------------------------------------------------------------------------
# SPDX-License-Identifier: Apache-2.0
# Copyright (C) 2024 Jayesh Badwaik <j.badwaik@fz-juelich.de>
# --------------------------------------------------------------------------------------------------

import jureap
import json


def test_log_01():

    input_file = "test/share/log/01.input.csv"
    output_file = "test/share/log/01.output.json"
    environment = "test/share/log/01.env.file"
    input_version = 1
    output_version = 1

    env_json = jureap.detail.env_file_to_dict(environment)
    jureap.log.log(input_file, output_file, env_json, input_version, output_version)

    with open(output_file, "r") as file:
        output = json.load(file)

    with open("test/share/log/01.expected.json", "r") as file:
        expected = json.load(file)

    assert output == expected


def test_log_02():

    input_file = "test/share/log/02.input.csv"
    output_file = "test/share/log/02.output.json"
    environment = "test/share/log/02.env.file"
    input_version = 1
    output_version = 1

    env_json = jureap.detail.env_file_to_dict(environment)
    jureap.log.log(input_file, output_file, env_json, input_version, output_version)

    with open(output_file, "r") as file:
        output = json.load(file)

    with open("test/share/log/02.expected.json", "r") as file:
        expected = json.load(file)

    assert output == expected


def test_log_03():

    input_file = "test/share/log/03.input.csv"
    output_file = "test/share/log/03.output.json"
    environment = "test/share/log/03.env.file"
    input_version = 1
    output_version = 1

    env_json = jureap.detail.env_file_to_dict(environment)
    jureap.log.log(input_file, output_file, env_json, input_version, output_version)

    with open(output_file, "r") as file:
        output = json.load(file)

    with open("test/share/log/03.expected.json", "r") as file:
        expected = json.load(file)

    assert output == expected



# --------------------------------------------------------------------------------------------------
# SPDX-License-Identifier: Apache-2.0
# Copyright (C) 2024 Jayesh Badwaik <j.badwaik@fz-juelich.de>
# --------------------------------------------------------------------------------------------------

import itertools
import os
import tempfile
import shutil
import subprocess
import json
import sys
import matplotlib.pyplot
import math
import numpy
import jureap.detail
import glob


class MachineData:
    def __init__(self, name):
        self.name = name
        self._node_array = []
        self._runtime_array = []
        self.is_empty = True

    def add_data(self, nodes, runtime):
        if len(nodes) != 0:
            self.is_empty = False
        self._node_array = self._node_array + nodes
        self._runtime_array = self._runtime_array + runtime
        self._runtime_array = [x for _, x in sorted(zip(self._node_array, self._runtime_array))]
        self._node_array = sorted(self._node_array)

    def is_empty(self):
        return self.is_empty

    def name(self):
        return self.name

    def node_array(self):
        return self._node_array

    def runtime_array(self):
        return self._runtime_array


class ProcessedData:
    def __init__(self, machine_data):
        self.machine_data = machine_data
        self._name = machine_data.name
        if self._name == "jurecadc":
            node_scale_factor = 0.5
        elif self._name == "jedi":
            node_scale_factor = 1
        elif self._name == "juwelsbooster":
            node_scale_factor = 0.5
        else:
            raise ValueError("Unknown machine name")

        self._node_array = [p * node_scale_factor for p in machine_data.node_array()]
        self._runtime_array = machine_data.runtime_array()

    def name(self):
        return self._name

    def node_array(self):
        return self._node_array

    def runtime_array(self):
        return self._runtime_array

    def ideal_scaling(self):
        return [self._runtime_array[0] / p * self._node_array[0] for p in self._node_array]

    def low_scaling(self):
        return [p * 1.25 for p in self.ideal_scaling()]


def experiment_to_data(experiment):
    nodes = [int(p["parameter"]["nodes"]) for p in experiment.json_data()["data"]]
    runtime = [float(p["runtime"]) for p in experiment.json_data()["data"]]
    return {"nodes": nodes, "runtime": runtime}


def aggregate_data(experiment_array):
    aggregate = {}
    aggregate["jurecadc"] = MachineData("jurecadc")
    aggregate["jedi"] = MachineData("jedi")
    aggregate["juwelsbooster"] = MachineData("juwelsbooster")

    for experiment in experiment_array:
        data = experiment_to_data(experiment)
        if experiment.json_data()["experiment"]["system"] == "jurecadc":
            aggregate["jurecadc"].add_data(data["nodes"], data["runtime"])
        elif experiment.json_data()["experiment"]["system"] == "jedi":
            aggregate["jedi"].add_data(data["nodes"], data["runtime"])
        elif experiment.json_data()["experiment"]["system"] == "juwelsbooster":
            aggregate["juwelsbooster"].add_data(data["nodes"], data["runtime"])

    return aggregate


def aggregate_to_processed_data(aggregate):
    processed_data = []
    for name, machine_data in aggregate.items():
        if not machine_data.is_empty:
            processed_data.append(ProcessedData(machine_data))

    return processed_data


def compute_plot_range_limits(processed_data):
    range_limits = {}
    range_limits["xlim"] = [sys.maxsize, 0]
    range_limits["ylim"] = [sys.maxsize, 0]
    range_limits["xticks"] = []
    for data in processed_data:
        range_limits["xlim"][0] = min(range_limits["xlim"][0], min(data.node_array()))
        range_limits["xlim"][1] = max(range_limits["xlim"][1], max(data.node_array()))
        range_limits["ylim"][0] = min(range_limits["ylim"][0], min(data.runtime_array()))
        range_limits["ylim"][1] = max(range_limits["ylim"][1], max(data.runtime_array()))
        range_limits["ylim"][0] = min(range_limits["ylim"][0], min(data.ideal_scaling()))
        range_limits["ylim"][1] = max(range_limits["ylim"][1], max(data.low_scaling()))
        range_limits["xticks"] = range_limits["xticks"] + data.node_array()

    range_limits["xlim"] = [0.75 * range_limits["xlim"][0], 1.25 * range_limits["xlim"][1]]
    range_limits["ylim"] = [0.9 * range_limits["ylim"][0], 1.1 * range_limits["ylim"][1]]
    range_limits["xticks"] = sorted(list(set(range_limits["xticks"])))

    # last_position = 0
    # reduced_node_array = [range_limits["xticks"][0]]
    # for i in range(len(range_limits["xticks"]) - 1):
    #    if range_limits["xticks"][i + 1] / range_limits["xticks"][i] > 1.1:
    #        reduced_node_array.append(range_limits["xticks"][i])

    # if max(range_limits["xticks"]) not in reduced_node_array:
    #    reduced_node_array.append(max(range_limits["xticks"]))

    # if reduced_node_array[-2] / reduced_node_array[-1] < 1.5:
    #    reduced_node_array.pop(-2)

    # range_limits["xticks"] = reduced_node_array

    range_limits["xticks"] = [0.5 if p == 0.5 else int(p) for p in range_limits["xticks"]]

    if math.log(range_limits["ylim"][0]) < 0 or math.log(range_limits["ylim"][1]) < 0:
        range_limits["yticks"] = numpy.linspace(
            range_limits["ylim"][0], range_limits["ylim"][1], num=8
        )
    else:
        range_limits["yticks"] = numpy.logspace(
            math.log(range_limits["ylim"][0], 2),
            math.log(range_limits["ylim"][1], 2),
            num=8,
            base=2,
        )
        range_limits["yticks"] = [int(p) for p in range_limits["yticks"]]

    return range_limits


def prepare_plotting_data(experiment_array):
    aggregate = aggregate_data(experiment_array)
    processed_data = aggregate_to_processed_data(aggregate)

    plotting_data = {}
    plotting_data["data"] = []
    plotting_data["plot_param"] = {}
    plotting_data["plot_param"]["range_limits"] = compute_plot_range_limits(processed_data)

    for data in processed_data:
        plotting_data["data"].append(
            {
                "name": data.name(),
                "nodes": data.node_array(),
                "runtime": data.runtime_array(),
                "ideal_scaling": data.ideal_scaling(),
                "low_scaling": data.low_scaling(),
            }
        )

    return plotting_data


def map_machine_name_to_plot_label(machine_name):
    if machine_name == "jurecadc":
        return "JURECA-DC"
    elif machine_name == "jedi":
        return "JEDI"
    elif machine_name == "juwelsbooster":
        return "JUWELS Booster"
    else:
        raise ValueError("Unknown machine name")


def generate_plot_pdf_file(plotting_data, output_dir):
    plot = matplotlib.pyplot.figure()
    marker_array = itertools.cycle(("o", "^", "s", "X"))
    cmap = matplotlib.colormaps["tab10"]
    colorgen = iter([cmap(i) for i in range(len(plotting_data["data"]))])

    ax = plot.subplots()
    ax.set_xlabel("Number of JEDI Nodes  (log-scale)")
    ax.set_ylabel("Runtime / s   (log-scale)")
    ax.set_yscale("log")
    ax.set_xscale("log")
    ax.margins(2)
    ax.set_xlim(plotting_data["plot_param"]["range_limits"]["xlim"])
    ax.set_ylim(plotting_data["plot_param"]["range_limits"]["ylim"])
    ax.grid(visible=True, which="major", axis="x", linestyle="-", linewidth=0.5, alpha=0.5)
    ax.tick_params(
        axis="both", which="minor", labelbottom=False, labelleft=False, bottom=False, left=False
    )

    ax_t = ax.secondary_xaxis("top")
    ax_t.minorticks_off()
    ax_t.set_xticks(plotting_data["plot_param"]["range_limits"]["xticks"])
    ax_t.set_xticklabels([2 * i for i in plotting_data["plot_param"]["range_limits"]["xticks"]])

    if "juwelsbooster" in [data["name"] for data in plotting_data["data"]] and "jurecadc" in [
        data["name"] for data in plotting_data["data"]
    ]:
        ax_t.set_xlabel("Number of JURECA/JUWELS Booster Nodes  (log-scale)")
    elif "juwelsbooster" in [data["name"] for data in plotting_data["data"]] and "jurecadc" not in [
        data["name"] for data in plotting_data["data"]
    ]:
        ax_t.set_xlabel("Number of JUWELS Booster Nodes  (log-scale)")
    else:
        ax_t.set_xlabel("Number of JURECA-DC Nodes  (log-scale)")

    ax.set_xlim(plotting_data["plot_param"]["range_limits"]["xlim"])
    ax.set_ylim(plotting_data["plot_param"]["range_limits"]["ylim"])
    ax.set_xticks(plotting_data["plot_param"]["range_limits"]["xticks"])
    ax.set_yticks(plotting_data["plot_param"]["range_limits"]["yticks"])
    ax.set_xticklabels(plotting_data["plot_param"]["range_limits"]["xticks"])
    ax.set_yticklabels(plotting_data["plot_param"]["range_limits"]["yticks"])
    ax.yaxis.set_major_formatter(matplotlib.ticker.StrMethodFormatter("{x:.1f}"))

    for data in plotting_data["data"]:
        color = next(colorgen)
        marker = next(marker_array)
        ax.plot(
            data["nodes"],
            data["runtime"],
            color=color,
            label=map_machine_name_to_plot_label(data["name"]),
            marker=marker,
            markersize=8,
        )

        ax.plot(
            data["nodes"],
            data["ideal_scaling"],
            color=color,
            linestyle="--",
            linewidth=0.6,
            alpha=1,
        )

        ax.plot(
            data["nodes"], data["low_scaling"], color=color, linestyle="--", linewidth=0.6, alpha=1
        )

    ax.plot(
        [0],
        [0],
        label="80% parallel efficiency intervals",
        alpha=1,
        ls="--",
        color="grey",
    )

    plot.legend(bbox_to_anchor=(0.58, 0.32))

    pdf_file = os.path.join(output_dir, "plot.pdf")
    png_file = os.path.join(output_dir, "plot.png")

    plot.savefig(pdf_file, bbox_inches="tight")
    plot.savefig(png_file, bbox_inches="tight")
    return "plot.pdf"


def generate_plot_tex_file(experiment_array, output_dir):
    plotting_data = prepare_plotting_data(experiment_array)
    pdf_filename = generate_plot_pdf_file(plotting_data, output_dir)

    plotfilename = os.path.join(output_dir, "plot.tex")

    with open(plotfilename, "w") as plotfile:
        plotfile.write("% This file was generated by jureap.\n")
        plotfile.write("\\exacbplot{" + pdf_filename + "}{Caption}\n")


def generate_csv_table_tex_file(experiment_array, output_dir):
    tablefilename = os.path.join(output_dir, "table.tex")
    with open(tablefilename, "w") as tablefile:
        tablefile.write("% This file was generated by jureap.\n")

        for experiment in experiment_array:
            csv_file = os.path.join(
                "data", experiment.output_pipeline_dir(), experiment.prefix() + ".csv"
            )
            tablefile.write("\\exacbtable{" + csv_file + "}{Caption}\n")


def generate_json_tex_file(experiment_array, output_dir):
    jsonfilename = os.path.join(output_dir, "json.tex")
    with open(jsonfilename, "w") as jsonfile:
        jsonfile.write("% This file was generated by jureap.\n")

        for experiment in experiment_array:
            json_file = os.path.join(
                "data", experiment.output_pipeline_dir(), experiment.prefix() + ".json"
            )
            jsonfile.write("\\lstinputlisting[caption=Caption]{" + json_file + "}\n")


def generate_author_tex_file(output_dir):
    authorfilename = os.path.join(output_dir, "author.tex")
    with open(authorfilename, "w") as authorfile:
        authorfile.write("% This file was generated by jureap.\n")
        authorfile.write("\\title{JEDI Evaluation Report}\n")


def compile_report_pdf(output_dir):
    subprocess.run(["make", "debug"], cwd=output_dir, env=os.environ)


def prepare_report_dir(output_dir, share_dir):
    texdir = os.path.join(share_dir, "jureap/tex/jedi")
    shutil.copytree(texdir, output_dir)


def write_json_data(experiment_array, output_dir):
    json_dir = os.path.join(output_dir, "json")
    os.makedirs(json_dir, exist_ok=True)
    for experiment in experiment_array:
        json_filepath = os.path.join(
            json_dir, experiment.pipeline() + "." + experiment.prefix() + ".json"
        )
        with open(json_filepath, "w") as jsonfile:
            json.dump(experiment.json_repr(), jsonfile, indent=4)


def copy_raw_data(input_dir, experiment_array, output_dir):
    data_dir = os.path.join(output_dir, "data")
    os.makedirs(data_dir, exist_ok=True)
    for experiment in experiment_array:
        output_experiment_reldir = experiment.pipeline() + str(".") + experiment.prefix()
        output_experiment_dir = os.path.join(data_dir, output_experiment_reldir)
        input_experiment_dir = os.path.join(input_dir, experiment.pipeline_dir())
        csv_filepath = os.path.join(input_experiment_dir, experiment.prefix() + ".csv")
        json_filepath = os.path.join(input_experiment_dir, experiment.prefix() + ".json")
        energy_dirpath = os.path.join(input_experiment_dir, experiment.prefix() + ".energy")
        os.makedirs(output_experiment_dir, exist_ok=True)
        if os.path.exists(energy_dirpath):
            shutil.copytree(energy_dirpath, os.path.join(output_experiment_dir, "energy"))
        shutil.copy(csv_filepath, output_experiment_dir)
        shutil.copy(json_filepath, output_experiment_dir)


def compute_node_energy(energy_dir):
    file_list = glob.glob(os.path.join(energy_dir, "energy.*.csv"))
    same_node_file_list = {}

    for file in file_list:
        split_file_string = os.path.basename(file).split(".")
        name = ".".join(split_file_string[0:1])
        pid = split_file_string[-2]
        node = ".".join(split_file_string[2:-2])
        if node not in same_node_file_list:
            same_node_file_list[node] = [file]
        else:
            same_node_file_list[node].append(file)

    total_energy = 0

    for node in same_node_file_list:
        file_data = jureap.detail.csv_file_to_array(same_node_file_list[node][0])
        sanitized_file_data = []
        for index, data in enumerate(file_data):
            data["sensor"] = data[""]
            del data[""]
            sanitized_file_data.append(data)

        for data in sanitized_file_data:
            if "pynvml" in data["sensor"]:
                total_energy = total_energy + float(data["0"])

    return total_energy


def extract_experiment_energy_info(energy_dir):
    experiment_energy_info = {}
    experiment_energy_info["node"] = []
    experiment_energy_info["energy"] = []
    for directory in os.listdir(energy_dir):
        node = int(directory)
        energy = compute_node_energy(os.path.join(energy_dir, directory))
        experiment_energy_info["node"].append(node)
        experiment_energy_info["energy"].append(energy)

    return experiment_energy_info


def generate_energy_plot(experiment_array, output_dir):
    experiment_energy_info_array = []
    for experiment in experiment_array:
        energy_dir = os.path.join(
            output_dir, "data", experiment.pipeline() + str(".") + experiment.prefix(), "energy"
        )
        experiment_energy_info = extract_experiment_energy_info(energy_dir)
        experiment_energy_info["system"] = experiment.json_data()["reporter"]["info"]["system"]
        experiment_energy_info_array.append(experiment_energy_info)

    generate_energy_plot_file(experiment_energy_info_array, output_dir)


def generate_energy_plot_file(experiment_energy_info_array, output_dir):
    plot = matplotlib.pyplot.figure()
    marker_array = itertools.cycle(("o", "^", "s", "X"))
    cmap = matplotlib.colormaps["tab10"]

    ax = plot.subplots()
    ax.set_xlabel("Number of JEDI Nodes  (log-scale)")
    ax.set_ylabel("Runtime / s   (log-scale)")
    ax.set_yscale("log")
    ax.set_xscale("log")
    ax.margins(2)
    ax.grid(visible=True, which="major", axis="x", linestyle="-", linewidth=0.5, alpha=0.5)
    ax.tick_params(
        axis="both", which="minor", labelbottom=False, labelleft=False, bottom=False, left=False
    )

    ax_t = ax.secondary_xaxis("top")
    ax_t.minorticks_off()
    ax.yaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter('%.2f'))

    min_val = 10**9
    max_val = 0
    for experiment in experiment_energy_info_array:
        if experiment["system"] == "jedi":
            ax.set_xticks(experiment["node"])
            ax.set_xticklabels(experiment["node"])
            ax_t.set_xticks(experiment["node"])
            ax_t.set_xticklabels([2 * i for i in experiment["node"]])

            ax.set_xlim([0.8 * min(experiment["node"]), 1.2 * max(experiment["node"])])

        min_val = min(min_val, min(experiment["energy"]))
        max_val = max(max_val, max(experiment["energy"]))

        sorted_energy = [x for _, x in sorted(zip(experiment["node"], experiment["energy"]))]
        sorted_node = sorted(experiment["node"])

        marker = next(marker_array)
        data = experiment
        ax.plot(sorted_node, sorted_energy, label=data["system"], marker=marker, markersize=8)

    ax.set_ylim([0.8 * min_val, 1.2 * max_val])
    yticks = numpy.logspace(
        math.log(min_val, 2),
        math.log(max_val, 2),
        num=8,
        base=2,
    )
    ax.set_yticks(yticks)
    ax.set_yticklabels(yticks)

    plot.legend(bbox_to_anchor=(0.58, 0.32))

    pdf_file = os.path.join(output_dir, "energy.pdf")
    png_file = os.path.join(output_dir, "energy.png")

    plot.savefig(pdf_file, bbox_inches="tight")
    plot.savefig(png_file, bbox_inches="tight")
    return "energy.pdf"


def generate(input_dir, experiment_array, output_dir, share_dir, tmp_dir):
    prepare_report_dir(output_dir, share_dir)
    copy_raw_data(input_dir, experiment_array, output_dir)
    # generate_energy_plot(experiment_array, output_dir)
    generate_plot_tex_file(experiment_array, output_dir)
    generate_csv_table_tex_file(experiment_array, output_dir)
    generate_json_tex_file(experiment_array, output_dir)
    write_json_data(experiment_array, output_dir)
    generate_author_tex_file(output_dir)
    compile_report_pdf(output_dir)



# --------------------------------------------------------------------------------------------------
# SPDX-License-Identifier: Apache-2.0
# Copyright (C) 2024 Jayesh Badwaik <j.badwaik@fz-juelich.de>
# --------------------------------------------------------------------------------------------------

import jureap.detail
import shutil
import itertools
import os
import tempfile
import subprocess
import json
import sys
import matplotlib.pyplot
import math
import numpy
import matplotlib.ticker


def compute_data_list(input_dir):
    data_name_list = []
    for file in os.listdir(input_dir):
        if file.endswith(".csv"):
            data_name_list.append(file.split(".")[0])

    return data_name_list


def extract_nodes_and_runtime(input_file):
    data = jureap.detail.csv_file_to_array(input_file)
    nodes = []
    runtime = []
    for row in data:
        nodes.append(int(row["nodes"]))
        runtime.append(float(row["runtime"]))

    runtime = [x for _, x in sorted(zip(nodes, runtime))]
    nodes = sorted(nodes)
    ideal_scaling_array = [runtime[0] / n * nodes[0] for n in nodes]
    low_scaling_array = [1.25 * i for i in ideal_scaling_array]

    return {
            "nodes": nodes,
            "runtime": runtime,
            "ideal_scaling": ideal_scaling_array,
            "low_scaling": low_scaling_array,
            }


def extract_data(input_dir):
    workload_factor = read_workload_factor(input_dir)
    print(workload_factor)
    min_workload_factor = min(workload_factor.values())
    for key in workload_factor:
        workload_factor[key] = workload_factor[key] / min_workload_factor

    plot_data = []
    for file in os.listdir(input_dir):
        if file.endswith(".csv"):
            data = extract_nodes_and_runtime(os.path.join(input_dir, file))
            name = file.rsplit(".", 1)[0]
            plot_data.append(
                    {
                        "name": file.rsplit(".", 1)[0],
                        "data": data,
                        "workload_factor": workload_factor[name],
                        }
                    )

    return plot_data


def read_workload_factor(input_dir):
    workload_factor = {}
    for file in os.listdir(input_dir):
        if file.endswith(".csv"):
            basename = file.rsplit(".", 1)[0]
            workload_filename = file.rsplit(".", 1)[0] + ".workload"
            with open(os.path.join(input_dir, workload_filename), "r") as f:
                workload_factor[basename] = float(f.read().strip())

    return workload_factor


def prepare_output_dir(input_dir, output_dir):
    if not os.path.exists(output_dir):
        os.makedirs(output_dir, exist_ok=True)
    shutil.copytree(input_dir, os.path.join(output_dir, "input"))


def compute_plot_info(plot_data):
    plot_info = {}
    node_range = [sys.maxsize, 0]
    runtime_range = [sys.float_info.max, 0]

    min_plot_name = plot_data[0]["name"]
    min_node = sys.maxsize

    node_array = []

    for keydata in plot_data:
        data = keydata["data"]
        name = keydata["name"]
        if min(data["nodes"]) < min_node:
            min_plot_name = name
            min_node = min(data["nodes"])
        node_array = node_array + data["nodes"]
        node_range[0] = min(node_range[0], min(data["nodes"]))
        node_range[1] = max(node_range[1], max(data["nodes"]))
        runtime_range[0] = min(runtime_range[0], min(data["runtime"]))
        runtime_range[1] = max(runtime_range[1], max(data["runtime"]))

    for plot in plot_data:
        if plot["name"] == min_plot_name:
            base_expected_runtime = plot["workload_factor"] * plot["data"]["runtime"][0]

    expected_runtime_array = {}
    for plot in plot_data:
        name = plot["name"]
        data = plot["data"]
        expected_runtime_array[name] = (
                base_expected_runtime * plot["workload_factor"] / data["nodes"][0] * min_node
                )

    node_range = [node_range[0] * 0.75, node_range[1] * 1.25]
    runtime_range = [runtime_range[0] * 0.75, runtime_range[1] * 1.12]

    node_array = sorted(list(set(node_array)))
    reduced_node_array = [node_array[0]]
    last_position = 0
    for i in range(len(node_array) - 1):
        if node_array[i + 1] / reduced_node_array[-1] > 1.5:
            reduced_node_array.append(node_array[i + 1])

    if max(node_array) not in reduced_node_array:
        reduced_node_array.append(max(node_array))

    if reduced_node_array[-2] / reduced_node_array[-1] > 0.66:
        reduced_node_array.pop(-2)
    node_array = reduced_node_array

    plot_info["xticklabels"] = node_array
    yticklabel_range = [math.log(runtime_range[0]), math.log(runtime_range[1])]
    plot_info["yticklabels"] = numpy.logspace(
            yticklabel_range[0], yticklabel_range[1], num=6, base=math.e
            )
    plot_info["yticklabels"] = [int(p) if p > 1 else p for p in plot_info["yticklabels"]]
    plot_info["min_plot_name"] = min_plot_name
    plot_info["range_limits"] = {"node": node_range, "runtime": runtime_range}

    plot_info["min_plot_name"] = min_plot_name
    plot_info["expected_runtime_array"] = expected_runtime_array
    return plot_info


def generate_plot_file(system_name, output_dir, plot_data, skip_weak_scaling, legend_position):
    plot = matplotlib.pyplot.figure()
    marker = itertools.cycle(("o", "^", "s", "X"))

    ax = plot.add_subplot(111, label="1")
    ax.ticklabel_format(useOffset=False)

    cmap = matplotlib.colormaps["tab10"]
    colorgen = iter([cmap(i) for i in range(len(plot_data))])
    plot_info = compute_plot_info(plot_data)

    error_yaxis = []
    xaxis_point = []
    yaxis_point = []

    color_history = []

    labeled_interval = False

    ax.fill_between(
            [0, 0],
            [0, 0],
            [0, 0],
            label="80% parallel efficiency intervals",
            alpha=0.2,
            ls="--",
            color="grey",
            )
    for data in plot_data:
        color = next(colorgen)
        color_history.append(color)
        ax.plot(
                data["data"]["nodes"],
                data["data"]["runtime"],
                marker=next(marker),
                color=color,
                )
        if not labeled_interval:
            labeled_interval = True
        else:
            label = None
        ax.fill_between(
                data["data"]["nodes"],
                data["data"]["ideal_scaling"],
                data["data"]["low_scaling"],
                alpha=0.2,
                ls="--",
                color=color,
                )
        if data["name"] != plot_info["min_plot_name"]:
            expected_runtime = plot_info["expected_runtime_array"][data["name"]]
            error_yaxis_positive = 0.25 * expected_runtime
            error_yaxis_negative = 0
            error_yaxis.append([error_yaxis_negative, error_yaxis_positive])
            xaxis_point.append(data["data"]["nodes"][0])
            yaxis_point.append(expected_runtime)

            error_yaxis_transpose = [[], []]
            for point in error_yaxis:
                for i in range(2):
                    error_yaxis_transpose[i].append(point[i])

            minimum_error_bar_ymin = min(
                    [yxp - yerr for yxp, yerr in zip(yaxis_point, error_yaxis_transpose[0])]
                    )
            max_error_bar_ymax = max(
                    [yxp + yerr for yxp, yerr in zip(yaxis_point, error_yaxis_transpose[1])]
                    )

            if minimum_error_bar_ymin < plot_info["range_limits"]["runtime"][0]:
                plot_info["range_limits"]["runtime"][0] = minimum_error_bar_ymin
            if max_error_bar_ymax > plot_info["range_limits"]["runtime"][1]:
                plot_info["range_limits"]["runtime"][1] = max_error_bar_ymax

            error_bar_ymin = min(min(yaxis_point), min(error_yaxis_transpose[0]))

    if len(plot_data) > 1:
        if not skip_weak_scaling:
            eb = ax.errorbar(
                    xaxis_point,
                    yaxis_point,
                    label="80% weak scaling efficiency intervals\n(estimated relative to first red datum)",
                    yerr=error_yaxis_transpose,
                    fmt="none",
                    capsize=4.0,
                    ls="-",
                    alpha=0.7,
                    )
            eb[-1][0].set_linestyle("dashed")

    ax.set_xscale("log")
    ax.set_yscale("log")

    ax.set_xlim(plot_info["range_limits"]["node"])
    ax.set_ylim(plot_info["range_limits"]["runtime"])

    ax.set_xticks(plot_info["xticklabels"])
    ax.set_yticks(plot_info["yticklabels"])
    ax.minorticks_off()

    ax_t = ax.secondary_xaxis("top")
    ax_t.tick_params(axis="x", labelrotation=90)
    ax_t.minorticks_off()
    ax_t.set_xticks(plot_info["xticklabels"])
    ax_t.set_xticklabels([4 * i for i in plot_info["xticklabels"]])
    ax_t.set_xlabel("Number of NVIDIA A100 GPUs (log-scale)")

    ax.set_xticklabels(plot_info["xticklabels"])
    ax.set_yticklabels(plot_info["yticklabels"])
    if plot_info["range_limits"]["runtime"][1] < 1 or plot_info["range_limits"]["runtime"][0] < 1:
        ax.yaxis.set_major_formatter(("{x:.3f}"))
    ax.tick_params(axis="x", labelrotation=90)

    if system_name == "juwelsbooster":
        system_name_string = "JUWELS Booster"
    elif system_name == "jupiter":
        system_name_string = "JUPITER"
    elif system_name == "juredadc":
        system_name_string = "JURECA DC"
    else:
        raise ValueError("Unknown system name: " + system_name)


    ax.set_xlabel("Number of " +  system_name_string + " Nodes (log-scale)")
    ax.set_ylabel("Runtime / s (log-scale)")
    plot.legend(loc="lower right", bbox_to_anchor=(legend_position[0], legend_position[1]))
    plot.savefig(os.path.join(output_dir, "plot.png"), bbox_inches="tight")
    plot.savefig(os.path.join(output_dir, "plot.pdf"), bbox_inches="tight")


def generate(system_name, input_dir, output_dir, skip_weak_scaling, legend_position):
    prepare_output_dir(input_dir, output_dir)
    plot_data = extract_data(input_dir)
    generate_plot_file(system_name, output_dir, plot_data, skip_weak_scaling, legend_position)



# --------------------------------------------------------------------------------------------------
# SPDX-License-Identifier: Apache-2.0
# Copyright (C) 2024 Jayesh Badwaik <j.badwaik@fz-juelich.de>
# --------------------------------------------------------------------------------------------------

import os
import tempfile
import shutil
import subprocess
import csv
import json
import jureap
import jureap.report.jedi
import jureap.report.jupiter
import jureap.report.scaling


def generate(
    report_type,
    input_dir,
    pipeline_array,
    prefix_array,
    workload_factor_array,
    output_dir,
    share_dir,
    tmp_dir,
    skip_weak_scaling,
    legend_position,
):
    experiment_array = parse_experiments(
        input_dir, pipeline_array, prefix_array, workload_factor_array
    )

    if report_type == "jedi":
        jureap.report.jedi.generate(input_dir, experiment_array, output_dir, share_dir, tmp_dir)
    if report_type == "jupiter":
        jureap.report.jupiter.generate(input_dir, experiment_array, output_dir, share_dir, tmp_dir)
    elif report_type == "scaling":
        jureap.report.scaling.generate(
            input_dir,
            experiment_array,
            output_dir,
            share_dir,
            tmp_dir,
            skip_weak_scaling,
            legend_position,
        )
    else:
        raise ValueError(f"Report type {report_type} not implemented yet.")


def parse_experiments(input_dir, pipeline_array, prefix_array, workload_factor_array):
    experiment_array = []
    for index, pipeline in enumerate(pipeline_array):
        pipeline_dir_array = [f for f in os.listdir(input_dir) if f.startswith(pipeline)]
        if len(pipeline_dir_array) == 0:
            raise ValueError(f"Pipeline {pipeline} not found in {input_dir}")
        elif len(pipeline_dir_array) > 1:
            raise ValueError(f"Multiple directories found for pipeline {pipeline} in {input_dir}")

        pipeline_reldir = pipeline_dir_array[0]
        pipeline_dir = os.path.join(input_dir, pipeline_reldir)
        prefix = prefix_array[index]
        workload_factor = workload_factor_array[index]
        csv_file = os.path.join(pipeline_dir, f"{prefix}.csv")
        json_file = os.path.join(pipeline_dir, f"{prefix}.json")

        csv_data = jureap.detail.csv_file_to_array(csv_file)
        with open(json_file) as json_filepath:
            json_data = json.load(json_filepath)

        experiment_array.append(
            experiment(pipeline_reldir, pipeline, prefix, workload_factor, csv_data, json_data)
        )

    return experiment_array


class experiment:
    def __init__(self, pipeline_dir, pipeline, prefix, workload_factor, csv_data, json_data):
        self._pipeline_dir = pipeline_dir
        self._pipeline = pipeline
        self._prefix = prefix
        self._workload_factor = workload_factor
        self._csv_data = csv_data
        self._json_data = json_data

    def pipeline_dir(self):
        return self._pipeline_dir

    def pipeline(self):
        return self._pipeline

    def prefix(self):
        return self._prefix

    def csv_data(self):
        return self._csv_data

    def output_pipeline_dir(self):
        return self._pipeline + "." + self._prefix

    def json_data(self):
        return self._json_data

    def workload_factor(self):
        return self._workload_factor

    def json_repr(self):
        return {
            "pipeline": self._pipeline,
            "prefix": self._prefix,
            "csv_data": self._csv_data,
            "json_data": self._json_data,
            "workload_factor": self._workload_factor,
        }

    def __repr__(self):
        return json.dumps(self.json_repr(), indent=4)

    def __str__(self):
        return self.__repr__()
